import 'package:data_connection_checker/data_connection_checker.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:m4e_sdk/m4e_sdk.dart';
import 'package:m4e_sdk/src/data/api/apis.dart';
import 'package:m4e_sdk/src/data/services/services.dart';
import 'package:matcher/matcher.dart';
import 'package:mockito/mockito.dart';

class MockPermissionApi extends Mock implements PermissionApi {}

class MockDataConnectionChecker extends Mock implements DataConnectionChecker {}

void main() {
  M4ePermissionService _m4ePermissionService;

  MockPermissionApi _mockPermissionApi;
  MockDataConnectionChecker _mockDataConnectionChecker;

  setUp(() {
    _mockPermissionApi = MockPermissionApi();
    _mockDataConnectionChecker = MockDataConnectionChecker();

    _m4ePermissionService = M4ePermissionService(
      permissionApi: _mockPermissionApi,
      connectionChecker: _mockDataConnectionChecker,
    );
  });

  final _encoding = <EntityWalletAccess>[
    EntityWalletAccess.see,
    EntityWalletAccess.view,
  ];

  final _m4ePermissionForm = M4ePermissionForm(
    subId: M4eUniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e0'),
    objId: M4eUniqueId.fromUniqueString('46e87882-4ff8-4770-9846-870fbbcc35e1'),
    type: M4eAccessPermissionType.ENTITY_WALLET,
    encoding: _encoding,
  );

  group('m4e-permission-test | ', () {
    group('setPermission', () {
      test('should verify device connectivity status', () async {
        // arrange
        when(_mockDataConnectionChecker.hasConnection)
            .thenAnswer((_) async => true);

        // act
        await _m4ePermissionService.setPermission(_m4ePermissionForm);

        // assert
        verify(_mockDataConnectionChecker.hasConnection);
      });

      group('isConnected', () {
        setUp(() {
          when(_mockDataConnectionChecker.hasConnection)
              .thenAnswer((_) async => true);
        });

        test('should return [M4eUniqueId] when call is successfull', () async {
          // assert
          when(_mockPermissionApi.setPermission(any))
              .thenAnswer((_) async => unitImpl);

          // act
          final _result =
              await _m4ePermissionService.setPermission(_m4ePermissionForm);

          // assert
          verify(_mockPermissionApi.setPermission(_m4ePermissionForm));
          expect(_result, equals(unitImpl));
        });

        test('should throw [ServerException] when call is unsuccessfull',
            () async {
          // arrange
          when(_mockPermissionApi.setPermission(any))
              .thenThrow(M4eExceptionMessages.kUndefinedException);

          // act
          final _call = _m4ePermissionService.setPermission;

          // assert
          expect(() => _call(_m4ePermissionForm),
              throwsA(TypeMatcher<M4eServerException>()));
        });
      });

      group('isDisconnected', () {
        setUp(() {
          when(_mockDataConnectionChecker.hasConnection)
              .thenAnswer((_) async => false);
        });

        test(
            'should throw a ServerException when no internet connection is detected',
            () async {
          // arrange
          when(_mockPermissionApi.setPermission(any))
              .thenThrow(M4eExceptionMessages.kNoInternetConnectionException);

          // act
          final _call = _m4ePermissionService.setPermission;

          // assert
          expect(() => _call(_m4ePermissionForm),
              throwsA(TypeMatcher<M4eServerException>()));
        });
      });
    });
  });
}
